@@ -1,4 +1,4 @@
-
+#include <stdarg.h>
 #include "EXTERN.h"
 #include "matrix.h"
 #include "perl.h"
@@ -31,6 +31,13 @@ int shrink_to_effective_size_m( MAT* inmat );
 int shrink_to_effective_size_v( VEC* invec );
 int shrink_to_effective_size_px( PERM* inper );
 
+int not_simple(pdl *a);
+
+int set_dims(pdl *outpdl, int nd,...);
+int perlside( pdl *outpdl , 
+						 int has_coerced, int moved_data, int changed_size, 
+						 int changed_dims  ,
+						 int nd, ... ) ;
 
 /****************** Convert meschach to pdl object ******************* 
 
@@ -71,6 +78,7 @@ pdl* mat2pdl(pdl* outpdl, MAT* inmat, int dont_free, int may_coerce
 		for(k=0;k<outpdl->ndims; k++)
 			printf("%d ",outpdl->dims[k]);
 		printf("\n");
+		p_pdl(outpdl);
 	}
 																/* set pdl->data, nvals, datatype */
 	
@@ -81,9 +89,9 @@ pdl* mat2pdl(pdl* outpdl, MAT* inmat, int dont_free, int may_coerce
 			printf("### Keep Matrix Base Pointer \n");
 																/* Keep matrix "base" pointer */
 		if( !dont_free ) { 
-			free ( outpdl->data );
 			if(mespdl_verbose) 
-				printf("### Freed Data\n");
+				printf("### Free pdl->data  %p \n",outpdl->data);
+			free( outpdl->data );
 		}
 												
 		changed_size= shrink_to_effective_size_m(inmat);
@@ -106,10 +114,18 @@ pdl* mat2pdl(pdl* outpdl, MAT* inmat, int dont_free, int may_coerce
 
 		needed_mem = m * n  ;
 		if( needed_mem != outpdl->nvals ) {
-			if(mespdl_verbose) printf("### Must Reallocate \n");
-			if( !dont_free ) free ( outpdl->data );
+
+			if( !dont_free ){
+				if(mespdl_verbose) 
+					printf("### Free pdl->data  %p \n",outpdl->data);
+				free( outpdl->data ); 
+			}
+
 			outpdl->data = 
 				malloc( needed_mem * pdl_howbig(outpdl->datatype) );
+			if(mespdl_verbose) 
+				printf("### Alloc pdl->data  %p \n",outpdl->data);
+
 			moved_data = 1;
 		}
 																/* Time to write a ".g" ? */
@@ -133,70 +149,22 @@ for(k=0; k<needed_mem; k++)                          \
 			croak("Cannot convert Real to pdl n.%i ",outpdl->datatype);  
 
 #undef loop_thru_pdl
-		
-		M_FREE(inmat);
+		if(mespdl_verbose)
+			printf("### Freeing (M_FREE) %p %p \n",inmat->me,*(inmat->me),inmat);
+		M_FREE(inmat); 
 	}
 	outpdl->nvals = m * n ;
 
 																/****** set pdl->ndims, ndims *****/
 
-	if( outpdl->ndims != 2 ) {
-		outpdl->ndims= 2;
-		free(outpdl->dims);
-		if(!( outpdl->dims= (int*)malloc(2*sizeof(int))) )
-			croak("(mat2pdl) Unable to allocate dims (ndims=%d)", 
-						outpdl->ndims);
-		outpdl->dims[0] = n;
-		outpdl->dims[1] = m;
-		changed_dims = 2;
-	} else 
-		if (outpdl->dims[0] != n || outpdl->dims[1] != m     ){
-			outpdl->dims[0] = n;
-			outpdl->dims[1] = m;
-			changed_dims = 1;
-		}
-
+	changed_dims = set_dims(outpdl,2,n,m);
 
 																/***** Modify the Perl variable *****/ 
 		
-	h= (HV*)SvRV((SV*)outpdl->sv); /* Coherence on perl side */
-		
-																/* Datatype */
-	if( has_coerced ) {						
-		if(mespdl_verbose) 
-			printf("### Has Coerced \n");
-		if( !( (dp= hv_fetch(h,"Datatype",8,0)) && (SvIOK(d=*dp))) )
-			croak(" Unable to get to Datatype for modification %p %i ",
-						dp, dp? SvTYPE(d):0) ;
-		sv_setiv( d, pdl_Real );
-	}
-																/* Data */
-	if( has_coerced || moved_data || changed_size ) {
-		if(mespdl_verbose) 
-			printf("### Has Moved/Coerced/Resized  Data \n");
-		if( !( (dp= hv_fetch(h,"Data",4,0)) && (SvPOK(d=*dp))) )
-			croak(" Unable to get to Data for modification %p %i ",
-						dp, dp? SvTYPE(d):0) ;
-		
-		SvPVX(d) = (char*) outpdl->data; /* Todo : Optimize : Useless if */
-																		 /* only changed_size  */  
-		SvLEN(d) =  m*n*pdl_howbig(outpdl->datatype);
-		SvCUR(d) =  m*n*pdl_howbig(outpdl->datatype);
-	}
 
-	if( changed_dims ) {					/* Dimension */
-		if(mespdl_verbose) printf("### Has Changed Dims \n");
-		/* Dims */
-		if( !( (dp= hv_fetch(h,"Dims",4,0)) && (SvTYPE(d=*dp) == SVt_RV)) )
-			croak(" Unable to get to Dims for modification %p %i",
-						dp, dp? SvTYPE(d):0);
-		e= (AV*)SvRV(d);
-		
-		if( changed_dims == 2 ) av_clear(e);
+	perlside( outpdl, has_coerced, moved_data, changed_size, 
+						changed_dims, 2, n, m );
 
-		av_store(e,0,newSViv(n));
-		av_store(e,1,newSViv(m));
-	}
 
 	return(outpdl);
 }
@@ -247,9 +215,9 @@ pdl* vec2pdl(pdl *outpdl, VEC *invec,
 			printf("### Keep Vector Base Pointer \n");
 																/* Keep matrix "base" pointer */
 		if( !dont_free ) { 
-			free ( outpdl->data );
 			if(mespdl_verbose) 
-				printf("### Freed Data\n");
+				printf("### Free pdl->data  %p \n",outpdl->data);
+			free( outpdl->data ); 
 		}
 
 		changed_size= shrink_to_effective_size_v(invec);
@@ -273,9 +241,16 @@ pdl* vec2pdl(pdl *outpdl, VEC *invec,
 		needed_mem = d  ;
 		if( needed_mem != outpdl->nvals ) {
 			if(mespdl_verbose) printf("### Must Reallocate \n");
-			if( !dont_free ) free ( outpdl->data );
+			if( !dont_free ){
+ 			  if(mespdl_verbose) 
+				  printf("### Free pdl->data  %p \n",outpdl->data);
+        free( outpdl->data ); 
+      }
 			outpdl->data = 	
 				malloc( needed_mem * pdl_howbig(outpdl->datatype) );
+			if(mespdl_verbose) 
+				printf("### Alloc pdl->data  %p \n",outpdl->data);
+
 			moved_data = 1;
 		}
 																/* Time to write a ".g" ? */
@@ -288,8 +263,10 @@ for(k=0; k<needed_mem; k++)           \
 
 #undef loop_thru_pdl_v
 
-		
-		V_FREE(invec);
+		if(mespdl_verbose)
+			printf("### Freeing (V_FREE) %p %p \n",invec->ve,invec);		
+		V_FREE(invec); 
+
 	}
 	
 	if ( outpdl->nvals != d ){
@@ -301,67 +278,18 @@ for(k=0; k<needed_mem; k++)           \
 
 																/****** set pdl->ndims, ndims *****/
 	if( ! *no_dim ) {
-		if( outpdl->ndims != 2 ) {
+		changed_dims = set_dims(outpdl,2,1,d);
 
-			outpdl->ndims= 2;
-			free(outpdl->dims);
-			if(!( outpdl->dims= (int*)malloc(2*sizeof(int))) )
-				croak("(vec2pdl) Unable to allocate dims (ndims=%d)", 
-							outpdl->ndims);
-			outpdl->dims[0] = 1;
-			outpdl->dims[1] = d;
-			changed_dims = 2;
-			
-		} else 
-			
-			if (outpdl->dims[1] != d ){
-				outpdl->dims[0] = 1;
-				outpdl->dims[1] = d;
-				changed_dims = 1;
-			}
 	}
 
+
+
 																/***** Modify the Perl variable *****/ 
 		
-	h= (HV*)SvRV((SV*)outpdl->sv); /* Coherence on perl side */
-		
-																/* Datatype */
-	if( has_coerced ) {						
-		if(mespdl_verbose) 
-			printf("### Has Coerced \n");
-		if( !( (ep= hv_fetch(h,"Datatype",8,0)) && (SvIOK(e=*ep))) )
-			croak(" Unable to get to Datatype for modification %p %i ",
-						ep, ep? SvTYPE(e):0) ;
-		sv_setiv( e, pdl_Real );
-	}
-																/* Data */
-	if( has_coerced || moved_data || changed_size ) {
-		if(mespdl_verbose) 
-			printf("### Has Moved/Coerced/Resized  Data \n");
-		if( !( (ep= hv_fetch(h,"Data",4,0)) && (SvPOK(e=*ep))) )
-			croak(" Unable to get to Data for modification %p %i ",
-						ep, ep? SvTYPE(e):0) ;
-		
-		SvPVX(e) = (char*) outpdl->data; /* Todo : Optimize : Useless if */
-																		 /* only changed_size  */  
-		SvLEN(e) =  d*pdl_howbig(outpdl->datatype);
-		SvCUR(e) =  d*pdl_howbig(outpdl->datatype);
-	}
 
-	if( changed_dims && ! *no_dim ) {					/* Dimension */
-		if(mespdl_verbose) printf("### Has Changed Dims \n");
-		/* Dims */
-		if( !( (ep= hv_fetch(h,"Dims",4,0)) && (SvTYPE(e=*ep) == SVt_RV)) )
-			croak(" Unable to get to Dims for modification %p %i",
-						ep, ep? SvTYPE(e):0);
-		f= (AV*)SvRV(e);
-		
-		if( changed_dims == 2 ) av_clear(f);
+	perlside( outpdl, has_coerced, moved_data, changed_size, 
+						changed_dims * ( *no_dim ? 0 : 1 )  , 2, 1, d );
 
-/* Code for row vector 		av_store(f,0,newSViv(d)); */
- 		av_store(f,0,newSViv(1));
-		av_store(f,1,newSViv(d)); 
-	}
 
 	if(mespdl_verbose) printf("vec2pdl ENDS \n\n");
 
@@ -404,9 +332,9 @@ pdl* perm2pdl(pdl* outpdl, PERM* inperm, int dont_free, int may_coerce
 			printf("### Keep Permutation Base Pointer \n");
 																/* Keep matrix "base" pointer */
 		if( !dont_free ) { 
-			free ( outpdl->data );
 			if(mespdl_verbose) 
-				printf("### Freed Data\n");
+				printf("### Free pdl->data  %p \n",outpdl->data);
+			free ( outpdl->data );
 		}
 
 		changed_size= shrink_to_effective_size_px(inperm);
@@ -431,9 +359,16 @@ pdl* perm2pdl(pdl* outpdl, PERM* inperm, int dont_free, int may_coerce
 		needed_mem = d  ;
 		if( needed_mem != outpdl->nvals ) {
 			if(mespdl_verbose) printf("### Must Reallocate \n");
-			if( !dont_free ) free ( outpdl->data );
+			if( !dont_free ) {
+				if(mespdl_verbose) 
+					printf("### Free outpdl->data  %p \n",outpdl->data);
+				free( outpdl->data );
+			}
 			outpdl->data = 	
 				malloc( needed_mem * pdl_howbig(outpdl->datatype) );
+			if(mespdl_verbose) 
+				printf("### Alloc pdl->data  %p \n",outpdl->data);
+
 			moved_data = 1;
 		}
 																/* Time to write a ".g" ? */
@@ -446,73 +381,26 @@ for(k=0; k<needed_mem; k++)           \
 
 #undef loop_thru_pdl_px
 
-		
+		if(mespdl_verbose) 
+			printf("### Free (PX_FREE)  %p \n",inperm->pe);
 		PX_FREE(inperm);
 	}
 	outpdl->nvals = d ;
 
 																/****** set pdl->ndims, ndims *****/
 
-	if( outpdl->ndims != 1 ) {
-		outpdl->ndims= 1;
-		free(outpdl->dims);
-		if(!( outpdl->dims= (int*)malloc(1*sizeof(int))) )
-			croak("(perm2pdl) Unable to allocate dims (ndims=%d)", 
-						outpdl->ndims);
-		outpdl->dims[0] = d;
-		changed_dims = 2;
-	} else 
-		if (outpdl->dims[0] != d ){
-			outpdl->dims[0] = d;
-			changed_dims = 1;
-		}
-
+	changed_dims = set_dims(outpdl,1,d);
 
 																/***** Modify the Perl variable *****/ 
-		
-	h= (HV*)SvRV((SV*)outpdl->sv); /* Coherence on perl side */
-		
-																/* Datatype */
-	if( has_coerced ) {
-		if(mespdl_verbose) 
-			printf("### Has Coerced \n");
-		if( !( (ep= hv_fetch(h,"Datatype",8,0)) && (SvIOK(e=*ep))) )
-			croak(" Unable to get to Datatype for modification %p %i ",
-						ep, ep? SvTYPE(e):0) ;
-		sv_setiv( e, outpdl->datatype );
-	}
-																/* Data */
-	if( has_coerced || moved_data || changed_size ) {
-		if(mespdl_verbose) 
-			printf("### Has Moved/Coerced/Resized  Data \n");
-		if( !( (ep= hv_fetch(h,"Data",4,0)) && (SvPOK(e=*ep))) )
-			croak(" Unable to get to Data for modification %p %i ",
-						ep, ep? SvTYPE(e):0) ;
-		
-		SvPVX(e) = (char*) outpdl->data; /* Todo : Optimize : Useless if */
-																		 /* only changed_size  */  
-		SvLEN(e) =  d*pdl_howbig(outpdl->datatype);
-		SvCUR(e) =  d*pdl_howbig(outpdl->datatype);
-	}
 
-	if( changed_dims ) {					/* Dimension */
-		if(mespdl_verbose) printf("### Has Changed Dims \n");
-		/* Dims */
-		if( !( (ep= hv_fetch(h,"Dims",4,0)) && (SvTYPE(e=*ep) == SVt_RV)) )
-			croak(" Unable to get to Dims for modification %p %i",
-						ep, ep? SvTYPE(e):0);
-		f= (AV*)SvRV(e);
-		
-		if( changed_dims == 2 ) av_clear(f);
-
-		av_store(f,0,newSViv(d));
-	}
+	perlside( outpdl, has_coerced, moved_data, changed_size, 
+						changed_dims  , 1, d );
 
 	return(outpdl);
-}
+}																/****** END perm2pdl ******/
 
 
-/******************** Convert pdl to meschch objects *****************
+/******************** Convert pdl to matrix objects *****************
 
 MAT*  pdl2mat ( pdl* inpdl , int *copy_copied )
 
@@ -546,6 +434,9 @@ MAT* pdl2mat( pdl* inpdl, int* copy_copied ) {
 
 	if( inpdl==(pdl*)NULL )	croak("inpdl is NULL");
 
+  if(not_simple(inpdl)!=0)
+		croak(" pdl2mat called with non-simple pdl ");
+
 	if( inpdl->ndims == 2 ) {
 
 		m= inpdl->dims[1];
@@ -608,10 +499,14 @@ MAT* pdl2mat( pdl* inpdl, int* copy_copied ) {
 
 		if( (outmat= (MAT*)malloc(sizeof(MAT))) == (MAT*)NULL )
 			croak("Unable to allocate a MAT ");
+		if(mespdl_verbose)
+			printf("### Alloc outmat  %p \n",outmat);
 
 		if( (outmat->me= (Real**) malloc( sizeof(Real*)*m*n )) == 
 			 (Real**)NULL )
 			croak("Unable to allocate outmat->me ");
+		if(mespdl_verbose)
+			printf("### Alloc outmat->me  %p \n",outmat->me);
 
 		for( i=0; i<m; i++ ) 
 			(outmat->me)[i]= ((Real*)inpdl->data) + n*i ;
@@ -624,8 +519,8 @@ MAT* pdl2mat( pdl* inpdl, int* copy_copied ) {
 	return(outmat);
 }
 
-/* Same as pdl2mat, but converts to a vector */
 
+/******** Same as pdl2mat, but converts to a vector ********/
 VEC* pdl2vec( pdl* inpdl, int* copy_copied ) {	
 
 	VEC *outvec;
@@ -643,6 +538,9 @@ VEC* pdl2vec( pdl* inpdl, int* copy_copied ) {
 
 	if( inpdl==(pdl*)NULL )	croak("inpdl is NULL in pdl2vec ");
 
+  if(not_simple(inpdl)!=0)
+		croak(" pdl2vec called with non-simple pdl ");
+
 	d= inpdl->nvals;
 
 	if(mespdl_verbose) printf(" d=%d \n",d); 
@@ -662,18 +560,24 @@ VEC* pdl2vec( pdl* inpdl, int* copy_copied ) {
 
 		outvec= v_get(d);
 
+		if(mespdl_verbose) 	printf("### START Looping \n");
+
 #define loop_thru_vec( X , T , Z)    \
 	for(i=0; i<X->dim; i++ )      \
 	     X->ve[i]= ((Real)(( (T*) Z->data )[i]))
 
 		all_types(loop_thru_vec,outvec,inpdl,inpdl->datatype);
-
 #undef loop_thru_vec
 
+		if(mespdl_verbose) 	printf("### Done Looping \n");
+		
+
 	} else {
 
 		if( (outvec= (VEC*)malloc(sizeof(VEC))) == (VEC*)NULL )
 			croak("Unable to allocate a VEC ");
+		if(mespdl_verbose)
+			printf("### Alloc outvec  %p \n",outvec);
 
 		outvec->dim= outvec->max_dim= d;
 		outvec->ve= inpdl->data;
@@ -695,6 +599,9 @@ PERM* pdl2perm( pdl* inpdl, int* copy_copied ) {
 
 	if( inpdl==(pdl*)NULL )	croak("inpdl is NULL in pdl2perm ");
 
+  if(not_simple(inpdl)!=0)
+		croak(" pdl2perm called with non-simple pdl ");
+
 	d= inpdl->nvals;
 	
 
@@ -718,6 +625,8 @@ PERM* pdl2perm( pdl* inpdl, int* copy_copied ) {
 
 		if( (outperm= (PERM*)malloc(sizeof(PERM))) == (PERM*)NULL )
 			croak("Unable to allocate a PERM ");
+		if(mespdl_verbose)
+			printf("### Alloc outperm  %p \n",outperm);
 
 		outperm->size= outperm->max_size= d;
 		outperm->pe= inpdl->data;
@@ -734,8 +643,12 @@ int mes_free_m(MAT* mat , int free_base ) {
 		return(1);
 
 	if ( free_base ) { 
+		if(mespdl_verbose)
+			printf("### Freeing (M_FREE) %p %p %p \n",mat->me,*(mat->me),mat);
 		M_FREE(mat);
 	} else {
+		if(mespdl_verbose)
+			printf("### Freeing (free) me= %p , mat=%p\n",mat->me,mat);
 		free(mat->me);
 		free(mat);
 		return(1);
@@ -747,9 +660,13 @@ int mes_free_v(VEC* vec, int free_ve ) {
 	if(vec==(VEC*)NULL)
 		return(1);
 	if( free_ve ) {
-		V_FREE(vec) ;
+		if(mespdl_verbose)
+			printf("### Freeing (V_FREE) ve=%p   vec=%p \n",vec->ve,vec);		
+		V_FREE(vec) ; 
 	} else {
-		free(vec);
+		if(mespdl_verbose)
+			printf("### Freeing (free) vec=%p \n",vec);		
+		free(vec); 
 	}
 	return(1);
 }
@@ -758,8 +675,12 @@ int mes_free_px(PERM* perm, int free_pe ) {
 	if(perm==(PERM*)NULL)
 		return(1);
 	if( free_pe ) {
+		if(mespdl_verbose)
+			printf("### Freeing (PX_FREE) pe=%p   perm=%p \n",perm->pe,perm);
 		PX_FREE(perm) ;
 	} else {
+		if(mespdl_verbose)
+			printf("### Freeing (free) perm=%p \n",perm);		
 		free(perm);
 	}
 	return(1);
@@ -785,8 +706,14 @@ int shrink_to_effective_size_m( MAT* inmat ) {
 		inmat->max_n= inmat->n;
 
 		if( inmat->m != inmat->max_m ){
+
+			if(mespdl_verbose)
+				printf("### Free inmat->me  %p \n",inmat->me);
 			free(inmat->me);
+
 			inmat->me= (Real**)malloc(sizeof(Real*) * inmat->m);
+			if(mespdl_verbose)
+				printf("### Alloc inmat->me  %p \n",inmat->me);
 		}
 		for(i=0;i<inmat->m; i++)
 			inmat->me[i]= inmat->base + i*inmat->n; 
@@ -829,4 +756,185 @@ int shrink_to_effective_size_px( PERM* inperm ) {
 	} else 
 		return(0);
 
+}
+
+
+																/* Check that pdl is a "trivial" one : */
+																/* Must correspond to consecutive */
+																/* memory locations. */ 
+int not_simple( pdl *a ){
+	int i,k;
+
+	/* HERE There's a problem; 
+     Remove next statement as soon as incs mystery clears.
+		 Probably with PDL-1.04. ... Nope, not yet.
+		 */
+	 return(0); 
+
+	for(k=0, i=1; k<a->ndims; k++){
+		if( a->incs[k] != i ){
+
+																/* For Debug Only */
+			printf(" non-simple : 	incs[%d]==%d!=%d\n",
+							 k,a->incs[k],i);
+ 			p_pdl(a);
+
+			return(k+1);
+		}
+		i *= a->dims[k];
+	}
+																/* For Debug Only */
+	if( a->offs) printf(" non-simple : offs : %d\n",
+																			 a->offs);
+
+	return( - a->offs );
+}
+
+
+																/* set pdl->ndims, dims, incs */
+int set_dims(pdl* outpdl,int nd,...){
+	va_list ap;
+	int *d, i,j, r;
+	
+	if( nd<=0 )
+		croak("set_dims with d==%d<=0 ",nd);
+
+	if( !(d= (int*)malloc(nd*sizeof(int))))
+		croak("set_dims unable to allocate d");
+	if(mespdl_verbose)
+		printf("### Alloc d  %p \n",d);
+
+	if(mespdl_verbose)  {
+		if(nd==1) printf("set_dims vector\n"); 
+		else if(nd==2)  printf("set_dims matrix\n"); 
+		fflush(stdout); 
+	}
+
+	r=0;
+
+	va_start(ap,nd);
+	if(mespdl_verbose)  printf(" d= ");
+	for(i=0; i<nd; i++){
+		d[i]= va_arg(ap,int);
+		if(mespdl_verbose)  {
+			printf(" %d ",d[i]); fflush(stdout); 
+		}
+	}
+	va_end(ap);
+	if(mespdl_verbose)  printf("\n");
+
+	if(outpdl->ndims != nd) {
+
+		r= 2;
+
+	} else {
+		
+		for( i=0; i<nd; i++ )
+			if (outpdl->dims[i] != d[i] ){
+				r= 1;
+				break;
+			}
+	}
+	if( r )	pdl_setdims(outpdl,d, nd, (int*)NULL) ;
+
+	if(mespdl_verbose)	printf("### Free d  %p \n", d);
+	free(d); 
+	
+	return(r);
+	
+
+}
+
+
+
+																/***** Modify the Perl variable *****/ 
+
+int perlside( pdl *outpdl , 
+						 int has_coerced, int moved_data, int changed_size, 
+						 int changed_dims,  
+						 int nd, ... ) {
+	va_list ap;
+	int *dim, i,j, sz;
+	HV *h;
+	SV *d, **dp;
+	AV *e;		
+		 
+	if(mespdl_verbose)  {
+		printf(" Perlside c%d   m%d   s%d    d%d    n%d \n",
+					 has_coerced, moved_data, changed_size, 
+					 changed_dims, nd); fflush(stdout);
+		fflush(stdout);
+		}
+
+	if( nd<=0 )
+		croak("set_dims with nd==%d<=0 ",nd);
+
+	if( !(dim= (int*)malloc(nd*sizeof(int))))
+		croak("set_dims unable to allocate dim");
+	if(mespdl_verbose)
+		printf("### Alloc dim  %p \n",dim);
+
+	va_start(ap,nd);
+	if(mespdl_verbose) printf(" dim = ");
+	for( sz=1 , i=0 ;   i<nd  ;  i++){ 
+		dim[i]= va_arg(ap,int);
+		sz*= dim[i];
+		if(mespdl_verbose)  {
+			printf(" %d ",dim[i]); fflush(stdout);
+		}
+	}
+	va_end(ap);
+	if(mespdl_verbose) printf("\n");
+
+	h= (HV*)SvRV((SV*)outpdl->sv); /* Coherence on perl side */
+
+	if( has_coerced ) {						
+		if(mespdl_verbose) 
+			printf("### Has Coerced \n");
+		if( !( (dp= hv_fetch(h,"Datatype",8,0)) && (SvIOK(d=*dp))) )
+			croak(" Unable to get to Datatype for modification %p %i ",
+						dp, dp? SvTYPE(d):0) ;
+		sv_setiv( d, outpdl->datatype );
+	}
+																/* Data */
+	if( has_coerced || moved_data || changed_size ) {
+		if(mespdl_verbose) 
+			printf("### Has Moved/Coerced/Resized  Data \n");
+		if( !( (dp= hv_fetch(h,"Data",4,0)) && (SvPOK(d=*dp))) )
+			croak(" Unable to get to Data for modification %p %i ",
+						dp, dp? SvTYPE(d):0) ;
+		
+		SvPVX(d) = (char*) outpdl->data; /* Todo : Optimize : Useless if */
+																		 /* only changed_size  */  
+		SvLEN(d) =  sz*pdl_howbig(outpdl->datatype);
+		SvCUR(d) =  sz*pdl_howbig(outpdl->datatype);
+	}
+
+	if( changed_dims ) {					/* Dimension */
+		if(mespdl_verbose) printf("### Has Changed Dims \n");
+		/* Dims */
+		if( !( (dp= hv_fetch(h,"Dims",4,0)) && (SvTYPE(d=*dp) == SVt_RV)) )
+			croak(" Unable to get to Dims for modification %p %i",
+						dp, dp? SvTYPE(d):0);
+		e= (AV*)SvRV(d);
+		
+		if( changed_dims == 2 ) av_clear(e);
+
+		for(i=0; i<nd; i++)
+			av_store(e,i,newSViv(dim[i]));
+
+		if(  (dp= hv_fetch(h,"Incs",4,0)) && (SvTYPE(d=*dp) == SVt_RV) )
+			  {
+					e= (AV*)SvRV(d);
+		
+					if( changed_dims == 2 ) av_clear(e);
+
+					for(i=0,j=1; i<nd; i++, j*=dim[i])
+						av_store(e,i,newSViv(dim[i]));
+				}
+
+	}
+	if(mespdl_verbose)
+		printf("### Free (free) dim=%p \n",dim);
+	free(dim); 
 }
